@@ -1,5 +1,5 @@
 /* Focus Blur -- blur with focus plug-in.
- * Copyright (C) 2002-2007 Kyoichiro Suda
+ * Copyright (C) 2002-2008 Kyoichiro Suda
  *
  * The GIMP -- an image manipulation program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
@@ -25,13 +25,17 @@
 #include <math.h>
 #include <stdlib.h>
 
-#include "focusblur.h"
 #include "focusblurparam.h"
 #include "diffusion.h"
+#include "brush.h"
 #ifdef HAVE_FFTW3
 #  include "fftblurbuffer.h"
 #endif
 
+#ifndef SQR
+#  define SQR(a) (a*a)
+#endif
+
 
 /*---- Types ----*/
 
@@ -64,6 +68,14 @@ static gfloat   focusblur_diffusion_model_flat
                                         (FblurDiffusionTable    *diffusion,
                                          gfloat                  radius,
                                          gfloat                 difference);
+static gfloat   focusblur_diffusion_model_spherical
+                                        (FblurDiffusionTable    *diffusion,
+                                         gfloat                  radius,
+                                         gfloat                 difference);
+static gfloat   focusblur_diffusion_model_gaussian
+                                        (FblurDiffusionTable    *diffusion,
+                                         gfloat                  radius,
+                                         gfloat                 difference);
 static gfloat   focusblur_diffusion_model_ring
                                         (FblurDiffusionTable    *diffusion,
                                          gfloat                  radius,
@@ -97,23 +109,44 @@ focusblur_diffusion_difference_table_init (void)
 }
 
 
-void
+gboolean
 focusblur_diffusion_update (FblurDiffusionTable **diffusion,
                             FblurFftBuffer       *fft,
-                            FblurStoreParam      *store)
+                            FblurStoreParam      *store,
+                            gchar                *brush_name)
 {
-  gfloat radius;
+  FblurBrush    *brush_tmp = NULL;
+  gsize          rowstride, centeroffset;
+  gint           radius_int, width;
+  gfloat         radius;
+  gfloat         density_max_bak = 0.0f;
 
   if (! difference_table)
     focusblur_diffusion_difference_table_init ();
-
+ 
   if (*diffusion)
     {
+      gboolean same_source, same_density, same_diffusion;
+      if (store->model_type == FBLUR_MODEL_BRUSH)
+        {
+          same_source = (*diffusion)->brush &&
+            focusblur_brush_name_is ((*diffusion)->brush, brush_name);
+          same_density = same_source &&
+            ! focusblur_brush_update_balance ((*diffusion)->brush,
+                                              store->enable_brush_balance);
+          same_diffusion = same_density &&
+            store->model_rotate == (*diffusion)->model_rotate;
+        }
+      else
+        same_source = same_density = same_diffusion = TRUE; /* ignored */
+
       if (store->model_type == (*diffusion)->model_type &&
           store->model_radius == (*diffusion)->model_radius &&
-          store->model_fill == (*diffusion)->model_fill &&
+          (store->model_type == FBLUR_MODEL_FLAT ||
+           store->model_type == FBLUR_MODEL_BRUSH ||
+           store->model_fill == (*diffusion)->model_fill) &&
           store->model_softness == (*diffusion)->model_softness &&
-          store->model_softness_delay == (*diffusion)->model_softness_delay)
+          same_diffusion /* model == brush */ )
         {
           if (store->shine_radius != (*diffusion)->shine_radius)
             {
@@ -122,37 +155,90 @@ focusblur_diffusion_update (FblurDiffusionTable **diffusion,
                 (*diffusion, (*diffusion)->shine_radius);
             }
 
-          return;
+          return TRUE;
+        }
+
+      if (store->shine_radius == (*diffusion)->shine_radius &&
+
+          store->model_type == (*diffusion)->model_type &&
+          same_density /* model == brush */ &&
+          (store->model_type == FBLUR_MODEL_BRUSH ||
+           store->model_type == FBLUR_MODEL_FLAT ||
+           store->model_fill == (*diffusion)->model_fill))
+        /* reuse */
+        density_max_bak = (*diffusion)->density_max;
+
+      if (same_source)
+        {
+          /* it doesn't need to update */
+          brush_tmp = (*diffusion)->brush;
+          (*diffusion)->brush = NULL;
         }
 
       focusblur_diffusion_destroy (diffusion);
     }
 
 #ifdef HAVE_FFTW3
-      if (fft)
-        focusblur_fft_buffer_invalidate_diffusion (fft);
+  if (fft)
+    focusblur_fft_buffer_invalidate_diffusion (fft);
 #endif
 
+  if (store->model_type == FBLUR_MODEL_BRUSH &&
+      ! brush_tmp)
+    {
+      brush_tmp =
+        focusblur_brush_new (brush_name, store->enable_brush_balance);
+
+      if (! brush_tmp)
+        return FALSE;
+    }
+
   *diffusion = g_new0 (FblurDiffusionTable, 1);
 
   (*diffusion)->model_type           = store->model_type;
   (*diffusion)->model_radius         = store->model_radius;
+  (*diffusion)->model_rotate         = store->model_rotate;
   (*diffusion)->model_fill           = store->model_fill;
   (*diffusion)->model_softness       = store->model_softness;
-  (*diffusion)->model_softness_delay = store->model_softness_delay;
   (*diffusion)->shine_radius         = store->shine_radius;
 
   (*diffusion)->model_fill_float = (*diffusion)->model_fill / 100.0f;
 
   radius = (*diffusion)->model_radius;
   radius *= 1.0f + (*diffusion)->model_softness / 100.0f;
-  (*diffusion)->model_radius_int = ceilf (radius);
-  (*diffusion)->rowstride = 1 + (*diffusion)->model_radius_int;
-  (*diffusion)->blocksize = sizeof (gfloat)
-    * (*diffusion)->rowstride * (1 + (*diffusion)->model_radius_int);
 
-  (*diffusion)->density_max = focusblur_diffusion_make_density
-    (*diffusion, (*diffusion)->shine_radius);
+  radius_int = ceilf (radius);
+
+  /* the figure symmetric on each axes */
+  if (store->model_type != FBLUR_MODEL_BRUSH)
+    {
+      width = 1 + radius_int;
+      rowstride = width;
+      centeroffset = 0; /* means symmetrical */
+    }
+  else
+    {
+      width = radius_int + 1 + radius_int;
+      rowstride = width;
+      centeroffset = rowstride * radius_int + radius_int;
+    }
+
+  (*diffusion)->model_radius_int = radius_int;
+  (*diffusion)->centeroffset = centeroffset;
+  (*diffusion)->rowstride = rowstride;
+  (*diffusion)->blocksize = sizeof (gfloat) * rowstride * width;
+  (*diffusion)->brush = brush_tmp;
+
+  if (density_max_bak)
+    (*diffusion)->density_max = density_max_bak;
+  else
+    (*diffusion)->density_max = focusblur_diffusion_make_density
+      (*diffusion, (*diffusion)->shine_radius);
+
+  g_assert ((*diffusion)->model_type != FBLUR_MODEL_BRUSH ||
+            (*diffusion)->brush      != NULL);
+
+  return TRUE;
 }
 
 
@@ -168,6 +254,8 @@ focusblur_diffusion_destroy (FblurDiffusionTable        **diffusion)
           g_slice_free1 ((*diffusion)->blocksize,
                          (*diffusion)->distribution[i]);
 
+      focusblur_brush_destroy (&((*diffusion)->brush));
+
       g_free (*diffusion);
       *diffusion = NULL;
     }
@@ -231,6 +319,44 @@ focusblur_diffusion_get (FblurDiffusionTable    *diffusion,
 }
 
 
+/* it is not enough for shrinked coordinates,
+   but it is not bad than to do nothing. */
+gfloat
+focusblur_diffusion_getf (FblurDiffusionTable   *diffusion,
+                          gint                   level,
+                          gfloat                 fx,
+                          gfloat                 fy)
+{
+  gint   x0, x1, y0, y1;
+  gfloat distribution, d[4];
+  gfloat xbal, ybal, r0, r1;
+
+  x0 = floorf (fx);
+  x1 = ceilf (fx);
+  y0 = floorf (fy);
+  y1 = ceilf (fy);
+
+  d[0] =
+    focusblur_diffusion_get (diffusion, level, x0, y0, 0, 0);
+  d[1] =
+    focusblur_diffusion_get (diffusion, level, x1, y0, 0, 0);
+  d[2] =
+    focusblur_diffusion_get (diffusion, level, x0, y1, 0, 0);
+  d[3] =
+    focusblur_diffusion_get (diffusion, level, x1, y1, 0, 0);
+
+  /* same as gimp_bilinear (fx, fy, d), but float. */
+
+  xbal = fx - x0;
+  ybal = fy - y0;
+  r0 = d[0] + (d[1] - d[0]) * xbal;
+  r1 = d[2] + (d[3] - d[2]) * xbal;
+  distribution = r0 + (r1 - r0) * ybal;
+
+  return distribution;
+}
+
+
 gfloat
 focusblur_diffusion_get_shine (FblurDiffusionTable      *diffusion,
                                gint                      depth_level,
@@ -286,34 +412,41 @@ focusblur_diffusion_make (FblurDiffusionTable   *diffusion,
   gfloat        *dlp, *dp;
   gfloat        *tlp, *tp;
   gfloat         radius;
+  gfloat         density;
   gfloat         fval;
-  gint           range;
+  gint           range, width;
   gint           x, y;
 
-  g_assert (level != 0);
+  g_assert (level > 0);
 
   radius = diffusion->model_radius;
 
   switch (diffusion->model_type)
     {
     case FBLUR_MODEL_FLAT:
-      //level = abs (level);
-      g_assert (level > 0);
       func = focusblur_diffusion_model_flat;
       break;
 
+    case FBLUR_MODEL_SPHERICAL:
+      func = focusblur_diffusion_model_spherical;
+      break;
+
+    case FBLUR_MODEL_GAUSSIAN:
+      func = focusblur_diffusion_model_gaussian;
+      break;
+
     case FBLUR_MODEL_RING:
-      //level = abs (level);
-      g_assert (level > 0);
       func = focusblur_diffusion_model_ring;
       break;
 
     case FBLUR_MODEL_CONCAVE:
-      //level = abs (level);
-      g_assert (level > 0);
       func = focusblur_diffusion_model_concave;
       break;
 
+    case FBLUR_MODEL_BRUSH:
+      func = 0;
+      break;
+
     default:
       g_assert_not_reached ();
       func = NULL; /* shut a warning */
@@ -321,48 +454,75 @@ focusblur_diffusion_make (FblurDiffusionTable   *diffusion,
 
   radius *= (gfloat) level / (gfloat) FBLUR_DEPTH_MAX;
   range = ceilf (radius);
+  width = 1 + range;
+  if (diffusion->centeroffset)
+    width += range;
 
   g_assert (radius >= 0.0f);
   g_assert (range < table_length);
 
   diffusion->distribution[num] = g_slice_alloc0 (diffusion->blocksize);
 
-  diffusion->density[num] = 0;
+  density = 0.0f;
   dlp = diffusion->distribution[num];
-  tlp = difference_table;
 
+  /* rendring and total value */
   if (diffusion->centeroffset == 0)
     {
-      for (y = 0; y <= range; y ++, dlp += diffusion->rowstride, tlp += table_length)
-        for (x = 0, dp = dlp, tp = tlp; x <= range; x ++, dp ++, tp ++)
-          diffusion->density[num] += (*dp = (*func) (diffusion, radius, *tp));
+      g_assert (func != 0);
+
+      tlp = difference_table;
+      for (y = width; y --; dlp += diffusion->rowstride, tlp += table_length)
+        for (x = width, dp = dlp, tp = tlp; x --; dp ++, tp ++)
+          density += (*dp = (*func) (diffusion, radius, *tp));
 
       /* 4 times density */
       dp = diffusion->distribution[num];
-      for (x = 0; x <= range; x ++, dp ++)
-        diffusion->density[num] -= *dp;
-      diffusion->density[num] *= 4;
-      diffusion->density[num] += diffusion->distribution[num][0];
+      for (x = width; x --; dp ++)
+        density -= *dp;
+      density *= 4;
+      density += diffusion->distribution[num][0];
     }
+  else
+    {
+      dlp += diffusion->centeroffset;
 
-  fval = 1.0f / diffusion->density[num];
-  dlp = diffusion->distribution[num];
+      switch (diffusion->model_type)
+        {
+        case FBLUR_MODEL_BRUSH:
+          g_assert (diffusion->brush != NULL);
+
+          density = focusblur_brush_render
+            (diffusion->brush, radius, diffusion->model_rotate,
+             diffusion->rowstride, dlp);
+          break;
+
+        default:
+          /* currently supports only brush */
+          g_assert_not_reached ();
+        }
+    }
+  diffusion->density[num] = MAX (density, 1.0f);
 
   /* normalize */
-  if (diffusion->centeroffset == 0)
-    {
-      for (y = 0; y <= range; y ++, dlp += diffusion->rowstride)
-        for (x = 0, dp = dlp; x <= range; x ++, dp ++)
-          *dp *= fval;
-    }
+  g_assert (density > 0.0f);
+  fval = 1.0f / density;
+  dlp = diffusion->distribution[num];
+  if (diffusion->centeroffset)
+    /* move to corner */
+    dlp += diffusion->centeroffset - diffusion->rowstride * range - range;
 
-  //diffusion->density[num] -= 1.0f; // make shine
+  for (y = width; y --; dlp += diffusion->rowstride)
+    for (x = width, dp = dlp; x --; dp ++)
+      *dp *= fval;
 
   focusblur_diffusion_blur
     (diffusion, diffusion->distribution[num], radius, level);
 }
 
 
+/* Pre-making figure and compute density. it limits
+   diffuseable maximum density to be shown as shining. */
 static gfloat
 focusblur_diffusion_make_density (FblurDiffusionTable   *diffusion,
                                   gfloat                 radius)
@@ -386,6 +546,14 @@ focusblur_diffusion_make_density (FblurDiffusionTable   *diffusion,
       func = focusblur_diffusion_model_flat;
       break;
 
+    case FBLUR_MODEL_SPHERICAL:
+      func = focusblur_diffusion_model_spherical;
+      break;
+
+    case FBLUR_MODEL_GAUSSIAN:
+      func = focusblur_diffusion_model_gaussian;
+      break;
+
     case FBLUR_MODEL_RING:
       func = focusblur_diffusion_model_ring;
       break;
@@ -394,6 +562,9 @@ focusblur_diffusion_make_density (FblurDiffusionTable   *diffusion,
       func = focusblur_diffusion_model_concave;
       break;
 
+    case FBLUR_MODEL_BRUSH:
+      return focusblur_brush_make_density (diffusion->brush, radius);
+
     default:
       g_assert_not_reached ();
       func = NULL; /* shut a warning */
@@ -409,19 +580,19 @@ focusblur_diffusion_make_density (FblurDiffusionTable   *diffusion,
 
   if (diffusion->centeroffset == 0)
     {
-      /* skip first line */
+      /* skip first line, it has been overlapped */
       tlp += table_length;
 
       for (y = 1; y <= range; y ++, tlp += table_length)
         for (x = 0, tp = tlp; x <= range; x ++, tp ++)
           density += (*func) (diffusion, radius, *tp);
 
-      /* 4 times (like a pinwheel) */
+      /* 4 times (like a pinwheel) and fill center */
       density *= 4;
       density += (*func) (diffusion, radius, 0.0f);
     }
-
-  //density -= 1.0f; // make shine
+  else
+    g_assert_not_reached ();
 
   return density;
 }
@@ -435,57 +606,49 @@ focusblur_diffusion_blur (FblurDiffusionTable   *diffusion,
 {
   const gint     width = 1 + 2 * FBLUR_RADIUS_MAX;
   gfloat         blur_table[width];
+  /* tp[] has minus offset. it reduces calling abs(). */
   gfloat        *tp = &(blur_table[FBLUR_RADIUS_MAX]);
   gfloat        *dlp, *dp, *sp;
 
-  gfloat         blur_radius;
-  gint           blur_radius_int;
   gint           range;
-  gfloat         vr, fval, sval;
+  gint           blur_radius_int;
   gint           x, y, i, j;
-
-  gfloat         softness, delay;
-  gint           delay_start, level_low;
+  gfloat         softness;
+  gfloat         blur_radius;
+  gfloat         vr, fval, sval;
 
   if (! diffusion->model_softness)
     return;
 
-  delay = 2.0f * diffusion->model_softness_delay / 100.0f;
-  if (delay <= 1.0f)
-    {
-      delay_start = (1.0f - delay);
-      level_low = 0 - 1;
-    }
-  else
-    {
-      delay_start = 0.0f;
-      level_low = rintf (FBLUR_DEPTH_MAX * (delay - 1.0f)) - 1;
-    }
-  if (level < level_low)
-    return;
   softness = diffusion->model_softness / 100.0f;
-  softness *= delay_start +
-    (1.0f - delay_start) * (level - level_low) / (FBLUR_DEPTH_MAX - level_low);
-
-
   range = diffusion->model_radius_int;
   blur_radius = model_radius * softness;
   blur_radius_int = ceilf (blur_radius);
 
-  vr = 0.3003866304f * (blur_radius + 1);
+  vr = FBLUR_GAUSSIAN_FACTOR * (blur_radius + 1);
   vr = -1.0f / (2.0f * SQR (vr));
+
+  /* make blurring table */
+  tp[0] = 1.0f;
   sval = 0.0f;
-  for (i = - blur_radius_int; i <= blur_radius_int; i ++)
+  for (i = 1; i <= blur_radius_int; i ++)
     {
       fval = expf (SQR (i) * vr);
-      tp[i] = fval;
+      tp[i] = tp[-i] = fval;
       sval += fval;
     }
+  sval *= 2.0f;
+  sval += 1.0f;
 
+  for (i = -blur_radius_int; i <= blur_radius_int; i ++)
+    tp[i] /= sval;
+
+  /* for symmetrical figures */
   if (diffusion->centeroffset == 0)
     {
       gfloat     sum[1 + range];
 
+      /* run on X lines */
       for (y = 0, dlp = data; y <= range; y ++, dlp += diffusion->rowstride)
         {
           for (x = 0, sp = sum; x <= range; x ++, sp ++)
@@ -493,16 +656,17 @@ focusblur_diffusion_blur (FblurDiffusionTable   *diffusion,
               fval = 0.0f;
               for (i = - blur_radius_int; i <= blur_radius_int; i ++)
                 {
-                  j = abs (x + i);
-                  if (j <= range)
+                  j = abs (x + i); /* abs() works as mirror */
+                  if (j <= range) /* limits inside of X */
                     fval += tp[i] * dlp[j];
                 }
-              *sp = fval / sval;
+              *sp = fval;
             }
           for (x = 0, dp = dlp, sp = sum; y <= range; y ++, dp ++, sp ++)
             *dp = *sp;
         }
 
+      /* run on Y lines */
       for (x = 0, dlp = data; x <= range; x ++, dlp ++)
         {
           for (y = 0, sp = sum; y <= range; y ++, sp ++)
@@ -514,16 +678,52 @@ focusblur_diffusion_blur (FblurDiffusionTable   *diffusion,
                   if (j <= range)
                     fval += tp[i] * dlp[j * diffusion->rowstride];
                 }
-              *sp = fval / sval;
+              *sp = fval;
             }
           for (y = 0, dp = dlp, sp = sum; y <= range;
                y ++, dp += diffusion->rowstride, sp ++)
             *dp = *sp;
         }
     }
-  else
+  else /* has center offset */
     {
-      // not implemented
+      gfloat     sum[range + 1 + range];
+
+      for (y = -range, dlp = data; y <= range;
+           y ++, dlp += diffusion->rowstride)
+        {
+          for (x = -range, sp = sum; x <= range; x ++, sp ++)
+            {
+              fval = 0.0f;
+              for (i = - blur_radius_int; i <= blur_radius_int; i ++)
+                {
+                  j = x + i;
+                  if (j >= -range && j <= range)
+                    fval += tp[i] * dlp[j];
+                }
+              *sp = fval;
+            }
+          for (x = -range, dp = dlp, sp = sum; y <= range; y ++, dp ++, sp ++)
+            *dp = *sp;
+        }
+
+      for (x = -range, dlp = data; x <= range; x ++, dlp ++)
+        {
+          for (y = -range, sp = sum; y <= range; y ++, sp ++)
+            {
+              fval = 0.0f;
+              for (i = - blur_radius_int; i <= blur_radius_int; i ++)
+                {
+                  j = y + i;
+                  if (j >= -range && j <= range)
+                    fval += tp[i] * dlp[j * diffusion->rowstride];
+                }
+              *sp = fval;
+            }
+          for (y = -range, dp = dlp, sp = sum; y <= range;
+               y ++, dp += diffusion->rowstride, sp ++)
+            *dp = *sp;
+        }
     }
 }
 
@@ -547,6 +747,77 @@ focusblur_diffusion_model_flat (FblurDiffusionTable     *diffusion,
 }
 
 
+static gfloat
+focusblur_diffusion_model_spherical (FblurDiffusionTable *diffusion,
+                                     gfloat               radius,
+                                     gfloat               difference)
+{
+  gfloat distribution;
+  gfloat base, r, nd, fd, nval, fval;
+
+  if (difference < 0.5f)
+    return 1.0f;
+
+  r = radius + 0.5f;
+  nd = difference - 0.5f;
+  fd = difference + 0.5f;
+
+  if (nd < 0.0f)
+    nd = 0.0f;
+  if (nd > r)
+    return 0.0f;
+
+  if (fd > r)
+    {
+      fd = r;
+      fval = 0.0f;
+    }
+  else
+    fval = sqrtf (1.0f - SQR (fd / r));
+
+  base = fd - nd;
+
+  nval = sqrtf (1.0f - SQR (nd / r));
+
+  distribution = base * (nval + fval) / 2.0f;
+
+  return distribution;
+}
+
+
+static gfloat
+focusblur_diffusion_model_gaussian (FblurDiffusionTable *diffusion,
+                                    gfloat               radius,
+                                    gfloat               difference)
+{
+  static gfloat cache_radius = 0.0f;
+  static gfloat cache_vr = 0.0f;
+
+  gfloat distribution;
+  gfloat vr;
+
+  if (difference - radius >= 1.0f)
+    return 0.0f;
+
+  if (radius == cache_radius)
+    {
+      vr = cache_vr;
+    }
+  else
+    {
+      vr = FBLUR_GAUSSIAN_FACTOR * (radius + 1);
+      vr = -1.0f / (2.0f * vr * vr);
+
+      cache_radius = radius;
+      cache_vr = vr;
+    }
+
+  distribution = expf (difference * difference * vr);
+
+  return distribution;
+}
+
+
 static gfloat
 focusblur_diffusion_model_ring (FblurDiffusionTable     *diffusion,
                                 gfloat                   radius,
@@ -554,6 +825,24 @@ focusblur_diffusion_model_ring (FblurDiffusionTable     *diffusion,
 {
   gfloat        distribution;
 
+  if (difference == 0.0f)
+    {
+      if (radius <= 5.0f)
+        distribution = 1.0f;
+
+      else if (radius < 1.0f)
+        {
+          distribution = 2.0f * (1.0f - radius);
+          distribution += diffusion->model_fill_float * (1.0f - distribution);
+
+          return distribution;
+        }
+      else
+        distribution = diffusion->model_fill_float;
+
+      return distribution;
+    }
+
   distribution = 1.0f + radius - difference;
 
   if (distribution <= 0.0f)
@@ -581,6 +870,24 @@ focusblur_diffusion_model_concave (FblurDiffusionTable  *diffusion,
 {
   gfloat        distribution;
 
+  if (difference == 0.0f)
+    {
+      if (radius <= 5.0f)
+        distribution = 1.0f;
+
+      else if (radius < 1.0f)
+        {
+          distribution = 2.0f * (1.0f - radius);
+          distribution += diffusion->model_fill_float * (1.0f - distribution);
+
+          return distribution;
+        }
+      else
+        distribution = diffusion->model_fill_float;
+
+      return distribution;
+    }
+
   distribution = 1.0f + radius - difference;
 
   if (distribution <= 0.0f)
